@@ -391,16 +391,16 @@ char            *item;
 *****************************************************************************/
 
 void    mpegaudio_mem_free(ptr_addr)
-void    **ptr_addr;
+void    *ptr_addr;
 {
 
-    if (*ptr_addr != NULL){
+    if (ptr_addr != NULL){
 #ifdef  MACINTOSH
-        DisposPtr(*ptr_addr);
+        DisposPtr(ptr_addr);
 #else
-        free(*ptr_addr);
+        free(ptr_addr);
 #endif
-        *ptr_addr = NULL;
+        ptr_addr = NULL;
     }
 
 }
