@@ -64,7 +64,7 @@ struct wave_header {
 
 static GstElementDetails gst_wavenc_details = {
   "WAV encoder",
-  "Codec/Encoder",
+  "Codec/Audio/Encoder",
   "LGPL",
   "Encode raw audio into WAV",
   VERSION,
@@ -136,8 +136,7 @@ gst_wavenc_change_state (GstElement *element)
   
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_PAUSED_TO_READY:
-    case GST_STATE_READY_TO_PAUSED:
-      wavenc->setup = FALSE;
+      wavenc->setup = FALSE;      
       wavenc->flush_header = TRUE;
       break;
     default:
@@ -207,7 +206,7 @@ gst_wavenc_setup (GstWavEnc *wavenc)
   return TRUE;
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_wavenc_sinkconnect (GstPad *pad,
 			GstCaps *caps)
 {
@@ -216,7 +215,7 @@ gst_wavenc_sinkconnect (GstPad *pad,
   wavenc = GST_WAVENC (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   }
 
   gst_caps_get_int (caps, "channels", &wavenc->channels);
@@ -226,10 +225,10 @@ gst_wavenc_sinkconnect (GstPad *pad,
   gst_wavenc_setup (wavenc);
 
   if (wavenc->setup) {
-    return GST_PAD_CONNECT_OK;
+    return GST_PAD_LINK_OK;
   }
 
-  return GST_PAD_CONNECT_REFUSED;
+  return GST_PAD_LINK_REFUSED;
 }
 
 static void
@@ -238,7 +237,7 @@ gst_wavenc_init (GstWavEnc *wavenc)
   wavenc->sinkpad = gst_pad_new_from_template (sinktemplate, "sink");
   gst_element_add_pad (GST_ELEMENT (wavenc), wavenc->sinkpad);
   gst_pad_set_chain_function (wavenc->sinkpad, gst_wavenc_chain);
-  gst_pad_set_connect_function (wavenc->sinkpad, gst_wavenc_sinkconnect);
+  gst_pad_set_link_function (wavenc->sinkpad, gst_wavenc_sinkconnect);
 
   wavenc->srcpad = gst_pad_new_from_template (srctemplate, "src");
   gst_element_add_pad (GST_ELEMENT (wavenc), wavenc->srcpad);
@@ -289,8 +288,30 @@ gst_wavenc_chain (GstPad *pad,
 	gst_pad_push (wavenc->srcpad, outbuf);
         wavenc->flush_header = FALSE;
       }
-    
-      gst_pad_push (wavenc->srcpad, buf);
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      /* RIFF only accepts little endian, so we need
+       * to do a little byte order conversion here */
+      if (wavenc->bits == 8) {
+#endif
+        gst_pad_push (wavenc->srcpad, buf);
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      } else {
+        GstBuffer *bsbuf = gst_buffer_new_and_alloc (GST_BUFFER_SIZE(buf));;
+        guint bps = (wavenc->bits + 7) / 8, i, byte;
+        guint8 *bsdata = GST_BUFFER_DATA(bsbuf), *data = GST_BUFFER_DATA (buf);
+
+        /* this is ugly... */
+        for (i = 0; i < GST_BUFFER_SIZE(bsbuf) / bps; i++) {
+          for (byte = 0; byte < bps; byte++) {
+            bsdata[i * bps + byte] = data[(i+1) * bps - (1+byte)];
+          }
+        }
+
+        gst_buffer_unref (buf);
+        gst_pad_push (wavenc->srcpad, bsbuf);
+      }
+#endif
     }
   }
 }
